@@ -1,7 +1,6 @@
-"""Smemo: Explicit session memoization
+"""Smemo: Explicit session memoization.
 
 Allow functions to be written naturally with explicit cache control.
-
 """
 
 import collections
@@ -20,7 +19,7 @@ ResType = typing.Tuple[typing.Any, typing.Optional[Exception]]
 
 
 PICKLED = object()
-"Marker object to identify pickled keys"
+"""Marker object to identify pickled keys."""
 
 
 PERSISTENT_FUNCS = set()  # type: typing.Set[FuncType]
@@ -29,7 +28,7 @@ PKEY_FUNCS = collections.defaultdict(
 
 
 class BaseSession:
-    "Session management interface"
+    """Session management interface."""
 
     def get_cache(self, func: FuncType, *args: typing.Any,
                   **kwargs: typing.Any) -> typing.Any:
@@ -45,47 +44,44 @@ class BaseSession:
 
     def pre_call(self, func: FuncType, args: PosType,
                  kwargs: KwdType) -> typing.Optional['BaseSession']:
-        """Called before actually checking cache
+        """Called before actually checking cache.
 
         This returns the session for making the call.  If it returns
         None, the call is skipped and it is treated as if the function
         returns None.
 
         Args:
-            func: The decorated function
-            args: The positional arguments for the call
-            kwargs: The keyword arguments for the call
-
+            func: The decorated function.
+            args: The positional arguments for the call.
+            kwargs: The keyword arguments for the call.
         """
         return None
 
     def cache_exc(self, func: FuncType, _exc: Exception,
                   *args: typing.Any, **kwargs: typing.Any) -> None:
-        """Insert an exception return as the cached value for a function call
+        """Insert an exception return as the cached value for a function call.
 
         Args:
-            func: The decorated function
-            _exc: The exception to cache
-            args: The positional arguments
-            kwargs: The keyword arguments
-
+            func: The decorated function.
+            _exc: The exception to cache.
+            args: The positional arguments.
+            kwargs: The keyword arguments.
         """
 
     def cache(self, func: FuncType, _val: typing.Any,
               *args: typing.Any, **kwargs: typing.Any) -> None:
-        """Insert a value as the cached value for a function call
+        """Insert a value as the cached value for a function call.
 
         Args:
-            func: The decorated function
-            _val: The value to cache
-            args: The positional arguments
-            kwargs: The keyword arguments
-
+            func: The decorated function.
+            _val: The value to cache.
+            args: The positional arguments.
+            kwargs: The keyword arguments.
         """
 
 
 class Session(BaseSession):
-    """Session object to hold cached results
+    """Session object to hold cached results.
 
     Args:
 
@@ -95,8 +91,7 @@ class Session(BaseSession):
 
         restrict: Restrict local cache to these functions.  If not
             None, functions not in restricted will always consult the
-            parent, for get_cache, cache and cache_exc operations
-
+            parent, for get_cache, cache and cache_exc operations.
     """
     def __init__(self, parent: BaseSession = None,
                  restrict: typing.Optional[typing.Iterable[FuncType]]
@@ -115,26 +110,24 @@ class Session(BaseSession):
 
     def setcache(self, ret: typing.Any = None,
                  exc: typing.Optional[Exception] = None) -> 'SetCacheSession':
-        """Get a SetCacheSession using the specified return value
+        """Get a SetCacheSession using the specified return value.
 
         Args:
-            ret: The return value if exc is None
-            exc: The exception to raise
-
+            ret: The return value if exc is None.
+            exc: The exception to raise.
         """
         return SetCacheSession(self, ret, exc)
 
     def getval(self, *args: typing.Any, **kwargs: typing.Any) -> typing.Any:
-        """Get values set by putval()
+        """Get values set by putval().
 
         The other arguments should match those of putval() exactly.
-
         """
         return getter(self, *args, **kwargs)
 
     def putval(self, val: typing.Any, *args: typing.Any,
                **kwargs: typing.Any) -> typing.Any:
-        """Put values to be used by getval()
+        """Put values to be used by getval().
 
         Use the session as a key-value store.  It actually uses the
         same caching mechanism to hold the data, so you can use any
@@ -142,10 +135,9 @@ class Session(BaseSession):
         value.
 
         Args:
-            val: The value to put
-            args: The arguments to identify the value
-            kwargs: The keyword arguments to identify the value
-
+            val: The value to put.
+            args: The arguments to identify the value.
+            kwargs: The keyword arguments to identify the value.
         """
         self.cache(getter, val, *args, **kwargs)
 
@@ -187,19 +179,18 @@ class Session(BaseSession):
 
     def invalidate(self, func: FuncType,
                    *args: typing.Any, **kwargs: typing.Any) -> None:
-        "Invalidate the result of a single function call"
+        """Invalidate the result of a single function call."""
         if func in self._cache:
             self._cache[func].pop(self._key(args, kwargs), None)
 
     def invalidate_all(self, func: FuncType = None) -> None:
-        """Invalidate all results, possibly restricted to a function call
+        """Invalidate all results, possibly restricted to a function call.
 
         If func is not specified, it invalidate all functions that are
         not persistent.
 
         Args:
-            func: The function to invalidate all cache
-
+            func: The function to invalidate all cache.
         """
         if func:
             self._cache.pop(func, None)
@@ -210,11 +201,10 @@ class Session(BaseSession):
             del self._cache[func]
 
     def invalidate_by_pkey(self, pkey: str) -> None:
-        """Invalidate all results for all function with a persistence key
+        """Invalidate all results for all function with a persistence key.
 
         Args:
-            pkey: The persistent key
-
+            pkey: The persistent key.
         """
         for func in set(self._cache):
             if func in PKEY_FUNCS.get(pkey, set()):
@@ -222,7 +212,7 @@ class Session(BaseSession):
 
     @contextlib.contextmanager
     def nocache(self) -> typing.Iterator[None]:
-        "Establish a context where caching is not performed"
+        """Establish a context where caching is not performed."""
         old_val = self._disabled
         self._disabled = True
         yield
@@ -243,7 +233,7 @@ class Session(BaseSession):
 
 
 class InvalidatorSession(BaseSession):
-    "Session which performs invalidation"
+    """Session which performs invalidation."""
     def __init__(self, session: Session) -> None:
         self._session = session
 
@@ -254,23 +244,22 @@ class InvalidatorSession(BaseSession):
 
 
 class CallOnlySession(BaseSession):
-    "Session which performs call only without caching"
+    """Session which performs call only without caching."""
     def __init__(self, session: Session) -> None:
         self._session = session
 
     def pre_call(self, func: FuncType,
                  args: PosType, kwargs: KwdType) -> typing.Any:
-        """Call a function without caching
+        """Call a function without caching.
 
         This normally calls a function directly, but may be overridden
         for other behavior.
-
         """
         return self._session
 
 
 class SetCacheSession(BaseSession):
-    "Session which causes a fixed value to be used"
+    """Session which causes a fixed value to be used."""
     def __init__(self, session: Session, ret: typing.Any,
                  exc: typing.Optional[Exception]) -> None:
         self._session = session
@@ -288,22 +277,21 @@ FuncT = typing.TypeVar('FuncT', bound=typing.Callable[..., typing.Any])
 
 
 def gcached(ref: bool = False, persistent: typing.Union[bool, str] = False) \
-        -> typing.Callable[[FuncType], FuncType]:
-    """Decorator to cache the return value of a function
+        -> typing.Callable[[FuncT], FuncT]:
+    """Decorator to cache the return value of a function.
 
     This is the generic decorator, and must be called instead of used
     directly as a decorator.  For such usage, use cached or rcached.
 
     Args:
 
-        ref: If False, copy.deepcopy is invoked before value is returned
+        ref: If False, copy.deepcopy is invoked before value is returned.
 
         persistent: If True, don't drop cached value upon
             invalidation.  If may be a string, which is called a
             "persistence key", or pkey, and can be used to
             mass-invalidate persistent functions of the same pkey in
-            the cache
-
+            the cache.
     """
     def _deco(func: FuncT) -> FuncT:
         @functools.wraps(func)
@@ -333,26 +321,32 @@ def gcached(ref: bool = False, persistent: typing.Union[bool, str] = False) \
     return _deco
 
 
-@gcached(ref=True, persistent=True)
-def getter(session: BaseSession, *args: typing.Any, **kwargs: typing.Any) \
-        -> typing.Any:
-    "The underlying function to get a value from the cache"
-    raise KeyError('No value cached for args %s %s' % (args, kwargs))
-
-
 def cached(func: FuncT) -> FuncT:
-    """Decorator to cache the return value of a function
-
-    Equivalent to gcached()(func)
+    """Decorator to cache the return value of a function.
 
+    Equivalent to gcached()(func).
     """
-    return gcached()(func)  # type: ignore
+    return gcached()(func)
 
 
 def rcached(func: FuncT) -> FuncT:
-    """Decorator to cache the return value of a function
-
-    Equivalent to gcached(ref=True)(func)
+    """Decorator to cache the return value of a function.
 
+    Equivalent to gcached(ref=True)(func).
     """
-    return gcached(ref=True)(func)  # type: ignore
+    return gcached(ref=True)(func)
+
+
+def prcached(func: FuncT) -> FuncT:
+    """Decorator to cache the return value of a function persistently.
+
+    Equivalent to gcached(ref=True, persistent=True)(func).
+    """
+    return gcached(ref=True, persistent=True)(func)
+
+
+@prcached
+def getter(session: BaseSession, *args: typing.Any, **kwargs: typing.Any) \
+        -> typing.Any:
+    """The underlying function to get a value from the cache."""
+    raise KeyError('No value cached for args %s %s' % (args, kwargs))
